@@ -1,18 +1,21 @@
 import os
 import subprocess
-import itertools
+import importlib
 from AppiumLibrary import AppiumLibrary
 from appium import webdriver
 from psutil import Process, NoSuchProcess
 from robot.api.deco import keyword
 from robot.libraries.BuiltIn import BuiltIn
-
 from selenium.common.exceptions import NoSuchElementException
 from selenium.webdriver.common.action_chains import ActionChains
-from time import sleep, time
+from time import sleep
+
+try:
+    AppiumCommon = importlib.import_module('Helpers.AppiumCommon', package='Helpers')
+except ModuleNotFoundError:
+    AppiumCommon = importlib.import_module('.Helpers.AppiumCommon', package='Zoomba')
 
 zoomba = BuiltIn()
-SCREENSHOT_COUNTER = itertools.count()
 
 
 class WinAppDriver:
@@ -104,9 +107,10 @@ class DesktopLibrary(AppiumLibrary):
 
     def get_keyword_names(self):
         """
-        This function restricts the keywords used in the library. This is to prevent incompatible keywords from imported
-        libraries from being referenced and used.
+        This function restricts the keywords used in the library. This is to prevent incompatible
+        keywords from imported libraries from being referenced and used.
         """
+        # self.save_appium_screenshot = save_appium_screenshot
         return [
             'maximize_window', 'open_application', 'wait_for_and_clear_text',
             'wait_for_and_click_element', 'wait_for_and_input_password', 'wait_for_and_input_text',
@@ -140,7 +144,7 @@ class DesktopLibrary(AppiumLibrary):
     def driver_setup(self, path=None):
         """Starts the WinAppDriver.
 
-        ``path`` can be provided if your winappdriver intallation is not in the default path of
+        ``path`` can be provided if your winappdriver installation is not in the default path of
         ``C:/Program Files (x86)/Windows Application Driver/WinAppDriver.exe``."""
         self.winappdriver.set_up_driver(path)
 
@@ -152,8 +156,7 @@ class DesktopLibrary(AppiumLibrary):
     @keyword("Maximize Window")
     def maximize_window(self):
         """Maximizes the current application window."""
-        driver = self._current_application()
-        driver.maximize_window()
+        self._current_application().maximize_window()
         return True
 
     @keyword("Open Application")
@@ -244,8 +247,9 @@ class DesktopLibrary(AppiumLibrary):
         """ Launch application. Application can be launched while Appium session running.
         This keyword can be used to launch application during test case or between test cases.
 
-        This keyword works while `Open Application` has a test running. This is good practice to `Launch Application`
-        and `Quit Application` between test cases. As Suite Setup is `Open Application`, `Test Setup` can be used to `Launch Application`
+        This keyword works while `Open Application` has a test running. This is good practice to
+        `Launch Application` and `Quit Application` between test cases. As Suite Setup is
+        `Open Application`, `Test Setup` can be used to `Launch Application`
 
         Example (syntax is just a representation, refer to RF Guide for usage of Setup/Teardown):
         | [Setup Suite] |
@@ -262,8 +266,7 @@ class DesktopLibrary(AppiumLibrary):
         See `Quit Application` for quiting application but keeping Appium session running.
         """
         self._open_desktop_session(self._current_application().command_executor)
-        driver = self._current_application()
-        driver.launch_app()
+        self._current_application().launch_app()
 
     @keyword("Wait For And Clear Text")
     def wait_for_and_clear_text(self, locator, timeout=None, error=None):
@@ -297,7 +300,8 @@ class DesktopLibrary(AppiumLibrary):
 
         See `introduction` for details about locating elements.
         
-        Use `Wait For And Mouse Over And Click Element` if this keyword gives issues in the application."""
+        Use `Wait For And Mouse Over And Click Element` if this keyword gives issues in the
+        application."""
         self._wait_until_page_contains_element(locator, timeout, error)
         self.click_element(locator)
 
@@ -311,8 +315,7 @@ class DesktopLibrary(AppiumLibrary):
 
         The difference between this keyword and `Wait For And Input Text` is that this keyword
         does not log the given password. See `introduction` for details about locating elements."""
-        self._wait_until_page_contains_element(locator, timeout, error)
-        self.input_password(locator, text)
+        AppiumCommon.wait_for_and_input_password(self, locator, text, timeout, error)
 
     @keyword("Wait For And Input Text")
     def wait_for_and_input_text(self, locator, text, timeout=None, error=None):
@@ -323,8 +326,7 @@ class DesktopLibrary(AppiumLibrary):
         ``error`` can be used to override the default error message.
 
         See `introduction` for details about locating elements."""
-        self._wait_until_page_contains_element(locator, timeout, error)
-        self.input_text(locator, text)
+        AppiumCommon.wait_for_and_input_text(self, locator, text, timeout, error)
 
     @keyword("Wait For And Long Press")
     def wait_for_and_long_press(self, locator, duration=10000, timeout=None, error=None):
@@ -335,8 +337,7 @@ class DesktopLibrary(AppiumLibrary):
         ``error`` can be used to override the default error message.
 
         See `introduction` for details about locating elements."""
-        self._wait_until_page_contains_element(locator, timeout, error)
-        self.long_press(locator, duration)
+        AppiumCommon.wait_for_and_long_press(self, locator, duration, timeout, error)
 
     @keyword("Wait Until Element Contains")
     def wait_until_element_contains(self, locator, text, timeout=None, error=None):
@@ -350,8 +351,7 @@ class DesktopLibrary(AppiumLibrary):
         `Wait Until Page Does Not Contain`
         `Wait Until Page Does Not Contain Element`
         """
-        self._wait_until_page_contains_element(locator, timeout, error)
-        self.element_should_contain_text(locator, text, error)
+        AppiumCommon.wait_until_element_contains(self, locator, text, timeout, error)
 
     @keyword("Wait Until Element Does Not Contain")
     def wait_until_element_does_not_contain(self, locator, text, timeout=None, error=None):
@@ -366,8 +366,7 @@ class DesktopLibrary(AppiumLibrary):
         `Wait Until Page Does Not Contain`
         `Wait Until Page Does Not Contain Element`
         """
-        self._wait_until_page_contains_element(locator, timeout, error)
-        self.element_should_not_contain_text(locator, text, error)
+        AppiumCommon.wait_until_element_does_not_contain(self, locator, text, timeout, error)
 
     @keyword("Wait Until Element Is Enabled")
     def wait_until_element_is_enabled(self, locator, timeout=None, error=None):
@@ -379,8 +378,7 @@ class DesktopLibrary(AppiumLibrary):
 
         See also `Wait Until Element Is Disabled`
         """
-        self._wait_until_page_contains_element(locator, timeout, error)
-        self.element_should_be_enabled(locator)
+        AppiumCommon.wait_until_element_is_enabled(self, locator, timeout, error)
 
     @keyword("Wait Until Element Is Disabled")
     def wait_until_element_is_disabled(self, locator, timeout=None, error=None):
@@ -392,8 +390,7 @@ class DesktopLibrary(AppiumLibrary):
 
         See also `Wait Until Element Is Disabled`
         """
-        self._wait_until_page_contains_element(locator, timeout, error)
-        self.element_should_be_disabled(locator)
+        AppiumCommon.wait_until_element_is_disabled(self, locator, timeout, error)
 
     @keyword("Mouse Over Element")
     def mouse_over_element(self, locator, x_offset=0, y_offset=0):
@@ -401,14 +398,14 @@ class DesktopLibrary(AppiumLibrary):
 
         ``x_offset`` and ``y_offset`` can be used to move to a specific coordinate.
         """
-        driver = self._current_application()
         element = self._element_find(locator, True, True)
-        actions = ActionChains(driver)
+        actions = ActionChains(self._current_application())
         self._move_to_element(actions, element, x_offset, y_offset)
         actions.perform()
 
     @keyword("Wait For And Mouse Over Element")
-    def wait_for_and_mouse_over_element(self, locator, timeout=None, error=None, x_offset=0, y_offset=0):
+    def wait_for_and_mouse_over_element(self, locator, timeout=None, error=None, x_offset=0,
+                                        y_offset=0):
         """Waits for and moves the mouse over the given ``locator``.
 
         Fails if ``timeout`` expires before the element appears.
@@ -443,8 +440,8 @@ class DesktopLibrary(AppiumLibrary):
         self.context_click_a_point()
 
     @keyword("Wait For And Mouse Over And Click Element")
-    def wait_for_and_mouse_over_and_click_element(self, locator, timeout=None, error=None, double_click=False,
-                                                  x_offset=0, y_offset=0):
+    def wait_for_and_mouse_over_and_click_element(self, locator, timeout=None, error=None,
+                                                  double_click=False, x_offset=0, y_offset=0):
         """Waits for, moves the mouse over, and clicks the given ``locator``.
 
         Fails if ``timeout`` expires before the element appears.
@@ -462,10 +459,10 @@ class DesktopLibrary(AppiumLibrary):
     def mouse_over_by_offset(self, x_offset=0, y_offset=0):
         """Moves the mouse from its current location by the given ``x_offset`` and ``y_offset``.
         """
-        driver = self._current_application()
-        actions = ActionChains(driver)
+        actions = ActionChains(self._current_application())
         actions.move_by_offset(x_offset, y_offset)
-        self._info('Moving mouse from current location with an offset of (%s,%s).' % (x_offset, y_offset))
+        self._info('Moving mouse from current location with an '
+                   'offset of (%s,%s).' % (x_offset, y_offset))
         actions.perform()
 
     @keyword("Click A Point")
@@ -476,16 +473,15 @@ class DesktopLibrary(AppiumLibrary):
 
         ``double_click`` can be used to click twice.
         """
-        driver = self._current_application()
-        actions = ActionChains(driver)
+        actions = ActionChains(self._current_application())
         if x_offset != 0 or y_offset != 0:
             actions.move_by_offset(x_offset, y_offset)
+        self._info("Clicking on current mouse position with an "
+                   "offset of (%s,%s)." % (x_offset, y_offset))
         if double_click:
-            actions.double_click()
+            actions.double_click().perform()
         else:
-            actions.click()
-        self._info("Clicking on current mouse position with an offset of (%s,%s)." % (x_offset, y_offset))
-        actions.perform()
+            actions.click().perform()
 
     @keyword("Context Click A Point")
     def context_click_a_point(self, x_offset=0, y_offset=0):
@@ -493,34 +489,25 @@ class DesktopLibrary(AppiumLibrary):
 
         ``x_offset`` and ``y_offset`` can be applied to give an offset.
         """
-        driver = self._current_application()
-        actions = ActionChains(driver)
+        actions = ActionChains(self._current_application())
         if x_offset != 0 or y_offset != 0:
             actions.move_by_offset(x_offset, y_offset)
-        actions.context_click()
-        self._info("Right-clicking on current mouse position with an offset of (%s,%s)." % (x_offset, y_offset))
-        actions.perform()
+        self._info("Right-clicking on current mouse position with an "
+                   "offset of (%s,%s)." % (x_offset, y_offset))
+        actions.context_click().perform()
 
     @keyword("Drag And Drop")
     def drag_and_drop(self, source, target):
-        """Drags the element found with the locator ``source`` to the element found with the locator ``target``.
-        """
-        driver = self._current_application()
-        source_element = self._element_find(source, True, True)
-        target_element = self._element_find(target, True, True)
-        actions = ActionChains(driver)
-        self._info('Dragging source element "%s" to target element "%s".' % (source, target))
-        actions.drag_and_drop(source_element, target_element).perform()
+        """Drags the element found with the locator ``source`` to the element found with the
+        locator ``target``."""
+        AppiumCommon.drag_and_drop(self, source, target)
 
     @keyword("Drag And Drop By Offset")
     def drag_and_drop_by_offset(self, locator, x_offset=0, y_offset=0):
-        """Drags the element found with ``locator`` to the given ``x_offset`` and ``y_offset`` coordinates.
+        """Drags the element found with ``locator`` to the given ``x_offset`` and ``y_offset``
+        coordinates.
         """
-        driver = self._current_application()
-        element = self._element_find(locator, True, True)
-        actions = ActionChains(driver)
-        self._info('Dragging element "%s" by offset (%s, %s).' % (locator, x_offset, y_offset))
-        actions.drag_and_drop_by_offset(element, x_offset, y_offset).perform()
+        AppiumCommon.drag_and_drop_by_offset(self, locator, x_offset, y_offset)
 
     @keyword("Send Keys")
     def send_keys(self, *argv):
@@ -529,12 +516,12 @@ class DesktopLibrary(AppiumLibrary):
         A list of special key codes can be found
         [https://seleniumhq.github.io/selenium/docs/api/py/webdriver/selenium.webdriver.common.keys.html|here]
 
-        Note that when sending in a modifier key (Ctrl, Alt, Shift) you will need to send the key again to release it.
+        Note that when sending in a modifier key (Ctrl, Alt, Shift) you will need to send the key
+        again to release it.
         |  Send Keys  |      a              |    b   |
         |  Send Keys  |      \\ue00        |    p    |    \\ue00     |
         """
-        driver = self._current_application()
-        actions = ActionChains(driver)
+        actions = ActionChains(self._current_application())
         self._info('Sending keys to application')
         if argv:
             for each in argv:
@@ -550,12 +537,12 @@ class DesktopLibrary(AppiumLibrary):
         A list of special key codes can be found
         [https://seleniumhq.github.io/selenium/docs/api/py/webdriver/selenium.webdriver.common.keys.html|here]
 
-        Note that when sending in a modifier key (Ctrl, Alt, Shift) you will need to send the key again to release it.
+        Note that when sending in a modifier key (Ctrl, Alt, Shift) you will need to send the key
+        again to release it.
         |  Send Keys To Element  |     locator    |      a              |    b   |
         |  Send Keys To Element  |     locator    |      \\ue00        |    p    |    \\ue00     |
         """
-        driver = self._current_application()
-        actions = ActionChains(driver)
+        actions = ActionChains(self._current_application())
         element = self._element_find(locator, True, True)
         self._info('Sending keys to element "%s".' % locator)
         if argv:
@@ -581,24 +568,13 @@ class DesktopLibrary(AppiumLibrary):
 
         See `Save Appium Screenshot` for a screenshot that will be unique across reports
         """
-        path, link = self._get_screenshot_paths(filename)
-
-        if hasattr(self._current_application(), 'get_screenshot_as_file'):
-            self._current_application().get_screenshot_as_file(path)
-        else:
-            self._current_application().save_screenshot(path)
-
-        # Image is shown on its own row and thus prev row is closed on purpose
-        self._html('</td></tr><tr><td colspan="3"><a href="%s">'
-                   '<img src="%s" width="800px"></a>' % (link, link))
-        return link
+        return AppiumCommon.capture_page_screenshot(self, filename)
 
     @keyword("Save Appium Screenshot")
     def save_appium_screenshot(self):
-        """Takes a screenshot with a unique filename to be stored in Robot Framework compiled reports."""
-        timestamp = time()
-        filename = 'appium-screenshot-' + str(timestamp) + '-' + str(next(SCREENSHOT_COUNTER)) + '.png'
-        return self.capture_page_screenshot(filename)
+        """Takes a screenshot with a unique filename to be stored in Robot Framework compiled
+        reports."""
+        return AppiumCommon.save_appium_screenshot(self)
 
     @keyword("Select Element From ComboBox")
     def select_element_from_combobox(self, list_locator, element_locator, skip_to_desktop=False):
@@ -687,8 +663,10 @@ class DesktopLibrary(AppiumLibrary):
                 criteria = locator_parts[2].strip()
         return prefix, criteria
 
+    def _wait_until_page_contains(self, text, timeout=None, error=None):
+        """Internal version to avoid duplicate screenshots"""
+        AppiumCommon.wait_until_page_contains(self, text, timeout, error)
+
     def _wait_until_page_contains_element(self, locator, timeout=None, error=None):
         """Internal version to avoid duplicate screenshots"""
-        if not error:
-            error = "Element '%s' did not appear in <TIMEOUT>" % locator
-        self._wait_until(timeout, error, self._is_element_present, locator)
+        AppiumCommon.wait_until_page_contains_element(self, locator, timeout, error)
